from fobi.constants import SUBMIT_VALUE_AS_REPR

__title__ = 'fobi.contrib.plugins.form_elements.fields.' \
            'checkbox_select_multiple.defaults'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('SUBMIT_VALUE_AS',)


SUBMIT_VALUE_AS = SUBMIT_VALUE_AS_REPR
